<?php

namespace Drupal\book_pdfs\EventSubscriber;

use Drupal\book\BookExport;
use Drupal\book_pdfs\Event\BookPdfsEvents;
use Drupal\book_pdfs\Event\BookPdfsPreGenerateEvent;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Link;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Book PDFs event subscriber.
 */
class BookPdfsPreGenerateSubscriber implements EventSubscriberInterface {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The Book export service.
   *
   * @var \Drupal\book\BookExport
   */
  protected $bookExport;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The book_pdfs logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The book_pdfs settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * Constructs event subscriber.
   *
   * @param \Drupal\book\BookExport $bookExport
   *   The Book export service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The date formatter service.
   */
  public function __construct(BookExport $bookExport, RendererInterface $renderer, LoggerChannelFactoryInterface $loggerFactory, RequestStack $requestStack, ConfigFactoryInterface $configFactory, DateFormatter $dateFormatter) {
    $this->bookExport = $bookExport;
    $this->renderer = $renderer;
    $this->logger = $loggerFactory->get('book_pdfs');
    $this->requestStack = $requestStack;
    $this->settings = $configFactory->get('book_pdfs.settings');
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * Adds the main book body to the PDF being generated.
   *
   * @param \Drupal\book_pdfs\Event\BookPdfsPreGenerateEvent $event
   *   The Book Pdf Pre Generate Event Object.
   *
   * @return \Drupal\book_pdfs\Event\BookPdfsPreGenerateEvent|false
   *   Event Object.
   */
  public function addBookHtml(BookPdfsPreGenerateEvent $event) {
    // Get the selected date format from the configuration.
    $date_format = $this->settings->get('date_format');
    $custom_date_format = $this->settings->get('custom_date_format') ?? 'F j, Y';

    // Determine the format to use.
    if ($date_format === 'custom') {
      $current_date = $this->dateFormatter->format(time(), 'custom', $custom_date_format);
    }
    else {
      // Use one of Drupal's built-in date formats.
      $current_date = $this->dateFormatter->format(time(), $date_format);
    }

    // Get the Book entity.
    $book = $event->getBook();
    $langcode = $event->getLangcode();
    if ($book->hasTranslation($langcode)) {
      $book = $book->getTranslation($langcode);
    }

    // Get the TOC mode setting.
    $toc_mode = $this->settings->get('toc_mode');

    try {
      // Get the header, footer, and splash page content from the config.
      $header = $this->settings->get('header') ?? '';
      $footer = $this->settings->get('footer') ?? '';
      $splash_page = $this->settings->get('splash_page') ?? '';

      $book_revision = $book->id();
      if ($book->getEntityType()->isRevisionable()) {
        // Get the revision ID if revisions are enabled.
        $book_revision = $book->getRevisionId();
      }
      $book_title = $book->getTitle();
      $book_url = Link::fromTextAndUrl($book_title, $book->toUrl('canonical', ['absolute' => TRUE]))->toString();

      $splash_page = $this->processSplashPlaceholders($splash_page, $book_revision, $book_title, $book_url, $current_date);
      $header = $this->processHeaderPlaceholders($header, $book_revision, $book_title, $book_url, $current_date);
      $footer = $this->processFooterPlaceholders($footer, $book_revision, $book_title, $book_url, $current_date);

      // Generate table of contents if TOC mode is enabled.
      $tableOfContentsHtml = '';
      if ($toc_mode) {
        $tableOfContentsBuild = $this->generateTableOfContents($book, $link_to_pdf = FALSE);
        $tableOfContentsHtml = $this->renderer->render($tableOfContentsBuild);
        $tableOfContentsHtml = '<div class="toc">' . $tableOfContentsHtml . '</div>';
        $base_path = $this->getDefaultBasePath($langcode);
        // Ensure the table of contents links open the expected web page.
        $tableOfContentsHtml = str_replace('a href="/' . $langcode, 'a href="' . $base_path . "/$langcode", $tableOfContentsHtml);
      }

      // Generate the main book content.
      $bookBuild = $this->bookExport->bookExportHtml($book);
      $bookHtml = $this->renderer->render($bookBuild);

      $html = $event->getHtml();
      // Check if custom CSS is enabled in the configuration.
      $css_enabled = $this->settings->get('css_enabled');
      $css_file = 'public://book_pdfs/book_pdfs.css';
      $file_exists = file_exists(\Drupal::service('file_system')->realpath($css_file));

      // If CSS is enabled and the CSS file exists, include it in the HTML.
      if ($css_enabled && $file_exists) {
        $css_content = file_get_contents(\Drupal::service('file_system')->realpath($css_file));
        $html = '<style>' . $css_content . '</style>';
      }
      // Prepare full content with optional splash page, table of contents, header, and footer.
      $html = '<meta charset="UTF-8"><style>
.splash-page { margin-bottom: 150px; padding-bottom: 100px; font-size: 20px; }
@page { margin-top: 100px; margin-bottom: 50px; }
header { position: fixed; top: -80px; left: 0px; right: 0px; height: 50px; font-size: 10px; }
footer { position: fixed; bottom: -40px; left: 0px; right: 0px; height: 30px; font-size: 10px; }
.pagenum:before { content: counter(page); }
.total-pages:before { content: counter(pages); }
</style>' . $html;

      // Add splash page if it's set.
      if (!empty($splash_page)) {
        $html .= '<div class="splash-page">' . $splash_page . '</div>';
      }

      // Add the header and footer.
      if (!empty($header)) {
        $html .= '<header>' . $header . '</header>';
      }
      if (!empty($footer)) {
        $html .= '<footer>' . $footer . '</footer>';
      }

      // Add table of contents and main book content.
      $html .= $toc_mode ? $tableOfContentsHtml . $bookHtml : $bookHtml;

      // Set the modified HTML back to the event.
      $event->setHtml($html);
    }
    catch (\Exception $e) {
      $this->logger->error('Error rendering Book export HTML. <br> %error', ['%error' => $e->getMessage()]);
      return FALSE;
    }

    return $event;
  }

  /**
   * Get the table of contents markup.
   *
   * @param \Drupal\node\Entity\Node $book
   *   The Book node.
   * @param boolean $link_to_self_mode
   *   The mode.
   *
   * @return array
   *   The render array for the table of contents.
   */
  protected function generateTableOfContents($book, $link_to_self_mode) {
    $bookPdfController = \Drupal::classResolver()->getInstanceFromDefinition('\Drupal\book_pdfs\Controller\BooktreeController');
    $table_of_contents = $bookPdfController->tree($book, $link_to_self_mode);
    return $table_of_contents;
  }

  /**
   * Get the default base path.
   *
   * @param string $langcode
   *   The langcode for the current lang.
   * @return string
   *   The default base path without the language prefix.
   */
  protected function getDefaultBasePath($langcode) {
    // Generate the absolute URL for the front page.
    $base_url = Url::fromRoute('<front>')->setAbsolute()->toString();

    $language_prefix = '/' . $langcode;

    // If the base URL ends with the language prefix, remove it.
    if (str_ends_with($base_url, $language_prefix)) {
      $base_url = str_replace($language_prefix, '', $base_url);
    }

    // Ensure the base path ends without a trailing slash.
    return rtrim($base_url, '/');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      BookPdfsEvents::PRE => ['addBookHtml'],
    ];
  }

  /**
   * Processes the footer placeholders and returns the final footer content.
   */
  protected function processFooterPlaceholders($footer, $book_revision, $book_title, $book_url, $current_date) {
    // Replace {{page}} with span elements that DomPDF will fill in.
    $footer = str_replace('{{book-revision}}', $book_revision, $footer);
    $footer = str_replace('{{book-url}}', $book_url, $footer);
    $footer = str_replace('{{book-title}}', $book_title, $footer);
    $footer = str_replace('{{date}}', $current_date, $footer);
    $footer = str_replace('{{page}}', '<span class="pagenum"></span>', $footer);

    return $footer;
  }

  /**
   * Processes the header placeholders and returns the final header content.
   */
  protected function processHeaderPlaceholders($header, $book_revision, $book_title, $book_url, $current_date) {
    // Replace {{page}} with span elements that DomPDF will fill in.
    $header = str_replace('{{book-revision}}', $book_revision, $header);
    $header = str_replace('{{book-url}}', $book_url, $header);
    $header = str_replace('{{book-title}}', $book_title, $header);
    $header = str_replace('{{date}}', $current_date, $header);
    $header = str_replace('{{page}}', '<span class="pagenum"></span>', $header);

    return $header;
  }

  /**
   * Processes the splash_page placeholders and returns the final header content.
   */
  protected function processSplashPlaceholders($splash_page, $book_revision, $book_title, $book_url, $current_date) {
    // Replace {{page}} with span elements that DomPDF will fill in.
    $splash_page = str_replace('{{book-revision}}', $book_revision, $splash_page);
    $splash_page = str_replace('{{book-url}}', $book_url, $splash_page);
    $splash_page = str_replace('{{book-title}}', $book_title, $splash_page);
    $splash_page = str_replace('{{date}}', $current_date, $splash_page);
    $splash_page = str_replace('{{page}}', '<span class="pagenum"></span>', $splash_page);

    return $splash_page;
  }

}

