<?php

namespace Drupal\book_pdfs\Form;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Url;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Book PDFs settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a SettingsForm object.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager service.
   */
  public function __construct(LanguageManagerInterface $language_manager) {
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('language_manager')
    );
  }

  /**
   * Get the default base path.
   *
   * @return string
   *   The default base path without the language prefix.
   */
  protected function getDefaultBasePath() {
    $language_manager = \Drupal::hasService('language_manager') ? \Drupal::languageManager() : NULL;

    // Generate the absolute URL for the front page.
    $base_url = Url::fromRoute('<front>')->setAbsolute()->toString();

    // Check if the language manager is available and multilingual is enabled.
    if ($language_manager && $language_manager->isMultilingual()) {
      $current_language = $language_manager->getCurrentLanguage();
      $language_prefix = '/' . $current_language->getId();

      // If the base URL ends with the language prefix, remove it.
      if (str_ends_with($base_url, $language_prefix)) {
        $base_url = str_replace($language_prefix, '', $base_url);
      }
    }

    // Ensure the base path ends without a trailing slash.
    return rtrim($base_url, '/');
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'book_pdfs_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['book_pdfs.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Add a link to the translation page using the route if config translation
    // is installed.
    if (\Drupal::moduleHandler()->moduleExists('config_translation')) {
      $form['translation_link'] = [
        '#type' => 'link',
        '#title' => $this->t('Translate Book PDFs Configuration'),
        '#url' => Url::fromRoute('config_translation.item.overview.book_pdfs.settings'),
        '#weight' => 1,
        '#attributes' => [
          'class' => ['button'],
          'target' => '_blank', // Optional: Opens the link in a new tab.
        ],
      ];
    }
    $config = $this->config('book_pdfs.settings');

    // Get the date formatter service.
    $date_formatter = \Drupal::service('date.formatter');

    // Fetch available date formats.
    $formats = \Drupal::entityTypeManager()->getStorage('date_format')->loadMultiple();
    $date_format_options = [];
    foreach ($formats as $format) {
      $date_format_options[$format->id()] = $format->label();
    }

    // Add a 'custom' option.
    $date_format_options['custom'] = $this->t('Custom');

    // Date format selection dropdown.
    $form['date_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Date format'),
      '#options' => $date_format_options,
      '#default_value' => $config->get('date_format') ?? 'custom',
      '#description' => $this->t('Select a date format for rendering the date in the PDF.'),
    ];

    // Custom date format input (only shown if "custom" is selected).
    $form['custom_date_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom date format'),
      '#default_value' => $config->get('custom_date_format') ?? 'F j, Y',
      '#description' => $this->t('Enter the custom PHP date format. This field is used only if "Custom" is selected above.'),
      '#states' => [
        'visible' => [
          ':input[name="date_format"]' => ['value' => 'custom'],
        ],
      ],
    ];

    // Get the default base path minus any language prefix if enabled.
    $default_base_path = $this->getDefaultBasePath();

    // Add the base path setting field.
    $form['base_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base Path for URLs'),
      '#description' => $this->t('The base path to prepend to URLs in the PDF. Leave empty to use the default base path.'),
      '#default_value' => $config->get('base_path') ?? '',
      '#placeholder' => $default_base_path,
    ];

    $form['basic_auth'] = [
      '#type' => 'fieldset',
      '#title' => 'Basic authentication',
      '#description' => 'Configure the Basic auth username and password phpwkhtmltopdf should use fetching remote assets while generating the PDF',
    ];
    $form['basic_auth']['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#default_value' => $config->get('basic_user'),
    ];
    $form['basic_auth']['password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#default_value' => $config->get('basic_pass'),
    ];

    // Add settings for booktree_deep and booktree_trim
    $form['booktree_deep'] = [
      '#type' => 'number',
      '#title' => $this->t('Booktree Depth'),
      '#default_value' => $config->get('booktree_deep') ?? 5,
      '#description' => $this->t('Maximum depth for the book tree.'),
      '#required' => TRUE,
    ];
    $form['booktree_trim'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Booktree Title Trim Length'),
      '#default_value' => $config->get('booktree_trim') ?? 35,
      '#description' => $this->t('Maximum length of the book titles in the tree.'),
      '#required' => TRUE,
    ];

    $form['file_prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('PDF file prefix'),
      '#default_value' => $config->get('file_prefix') ?? 'book-',
      '#description' => 'Optional. In the menu, the heavier items will sink and the lighter items will be positioned nearer the top.',
      '#element_validate' => [[static::class, 'validateFilePrefixElement']],
    ];

    // Add checkbox for TOC mode.
    $form['toc_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Table of Contents'),
      '#default_value' => $config->get('toc_mode') ?? FALSE,
      '#description' => $this->t('Enable or disable the Table of Contents in the generated PDF.'),
    ];

    $form['custom_css'] = array(
      '#type' => 'details',
      '#title' => t('Custom CSS'),
      '#open' => TRUE,
      '#tree' => TRUE,
    );

    // Switch to enable/disable customization.
    $form['custom_css']['css_enabled'] = array(
      '#title' => t('Enable or disable custom CSS:'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('css_enabled') ?? FALSE,
    );
    // Editor box.
    $form['custom_css']['css'] = array(
      '#type' => 'textarea',
      '#prefix' => '<div id="css-editor-field">',
      '#description' => t('Type or paste custom CSS code for this theme.'),
      '#default_value' => $config->get('css') ?? NULL,
      '#attributes' => array('id' => 'css-editor-textarea'),
      '#suffix' => '</div>',
    );
    // Switch to enable/disable the plain text editor.
    $form['custom_css']['plaintext_enabled'] = array(
      '#title' => t('Use plain text editor'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('plaintext_enabled') ?? FALSE,
      '#attributes' => [
        'id' => 'edit-plaintext-enabled',
      ],
    );
    $form['pdf_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('PDF Settings'),
      '#description' => $this->t('Configure PDF generation settings.'),
    ];

    // Add paper size selection.
    $form['pdf_settings']['paper_size'] = [
      '#type' => 'select',
      '#title' => $this->t('Paper Size'),
      '#options' => [
        'letter' => $this->t('Letter'),
        'a4' => $this->t('A4'),
        'legal' => $this->t('Legal'),
        // Add more sizes as needed.
      ],
      '#default_value' => $config->get('paper_size') ?? 'letter',
      '#description' => $this->t('Select the default paper size for the PDF.'),
    ];

    $form['header_footer'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Header and Footer'),
      '#description' => $this->t('Configure custom headers and footers for the PDF.'),
    ];
    // Header field (translateable).
    $form['header_footer']['header'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Header Content'),
      '#description' => $this->t('Define custom HTML for the header.  Use {{date}} for the current date.  Other tokens are {{book-title}} {{book-url}} {{book-revision}} {{page}}. Leave empty for no header.'),
      '#default_value' => $config->get('header') ?? '',
    ];
    // Footer field (translateable).
    $form['header_footer']['footer'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Footer Content'),
      '#description' => $this->t('Define custom HTML for the footer. Use {{date}} for the current date and use {{page}} for current page. Leave empty for no footer.  Other tokens are {{book-title}} {{book-url}} {{book-revision}}.'),
      '#default_value' => $config->get('footer') ?? '<hr><div style="text-align: left; float: left;">{{date}}</div><div style="text-align: right; float: right;">Page {{page}}</div>',
    ];
    // Splash Page field (translateable).
    $form['splash_page'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Splash Page Content'),
      '#description' => $this->t('HTML content to be used as a splash page at the beginning of the PDF. Leave empty for no splash page.  Available tokens: {{date}} {{page}} {{book-title}} {{book-url}} {{book-revision}}.'),
      '#default_value' => $config->get('splash_page') ?? '',
    ];

    // Add the "Delete PDF Cache" button.
    $form['delete_pdf_cache'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete PDF Cache'),
      '#submit' => ['::deletePdfCache'],
      '#description' => $this->t('This will delete all cached PDF files in the public://book_pdfs directory.'),
    ];

    // Attach CSS and Javascript libraries.
    $form['#attached']['library'][] = 'book_pdfs/css_editor';
    $form['#attached']['library'][] = 'book_pdfs/codemirror';
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save file.
    $path = 'public://book_pdfs';
    $file = $path . DIRECTORY_SEPARATOR . 'book_pdfs.css';
    if (\Drupal::service('file_system')->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
      \Drupal::service('file_system')->saveData($form_state->getValue(['custom_css', 'css']), $file, FileSystemInterface::EXISTS_REPLACE);
      \Drupal::service('file_system')->chmod($file);
    }
    $this->config('book_pdfs.settings')
      ->set('css_enabled', $form_state->getValue(['custom_css', 'css_enabled']))
      ->set('css', $form_state->getValue(['custom_css', 'css']))
      ->set('date_format', $form_state->getValue('date_format'))
      ->set('custom_date_format', $form_state->getValue('custom_date_format'))
      ->set('splash_page', $form_state->getValue('splash_page'))
      ->set('header', $form_state->getValue('header'))
      ->set('footer', $form_state->getValue('footer'))
      ->set('paper_size', $form_state->getValue('paper_size'))
      ->set('plaintext_enabled', $form_state->getValue(['custom_css', 'plaintext_enabled']))
      ->set('basic_user', $form_state->getValue('username'))
      ->set('basic_pass', $form_state->getValue('password'))
      ->set('file_prefix', $form_state->getValue('file_prefix'))
      ->set('booktree_deep', $form_state->getValue('booktree_deep'))
      ->set('booktree_trim', $form_state->getValue('booktree_trim'))
      ->set('toc_mode', $form_state->getValue('toc_mode'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Custom submit handler to delete cached PDF files.
   */
  public function deletePdfCache(array &$form, FormStateInterface $form_state) {
    $path = 'public://book_pdfs';
    $file_system = \Drupal::service('file_system');

    if ($file_system->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY)) {
      // Load all .pdf files in the directory.
      $pdf_files = $file_system->scanDirectory($path, '/\.pdf$/');

      if (!empty($pdf_files)) {
        foreach ($pdf_files as $file_info) {
          $file_system->delete($file_info->uri);
        }
        \Drupal::messenger()->addMessage($this->t('All cached PDF files have been deleted.'));
      }
      else {
        \Drupal::messenger()->addMessage($this->t('No cached PDF files found.'), 'warning');
      }
    }
    else {
      \Drupal::messenger()->addError($this->t('Failed to prepare the directory.'));
    }
  }

  /**
   * Form element validation handler for the 'file_prefix' element.
   *
   * Only allow letters, numbers, underscores and dashes.
   */
  public static function validateFilePrefixElement($element, FormStateInterface $form_state, $form) {
    $file_prefix = trim($element['#value']);
    $file_prefix = str_replace(' ', '-', $file_prefix);
    $form_state->setValueForElement($element, $file_prefix);

    if (!preg_match('/^[a-zA-Z0-9-_]+$/', $file_prefix)) {
      $form_state->setError($element, t('Allowed characters are a-z, 0-9, \'-\', and \'_\'.'));
      return;
    }
  }
}

